/*----------------------------------------------------------------------------
 *      Name:    DEMO.C
 *      Purpose: USB HID Demo
 *      Version: V1.10
 *----------------------------------------------------------------------------
 *      This software is supplied "AS IS" without any warranties, express,
 *      implied or statutory, including but not limited to the implied
 *      warranties of fitness for purpose, satisfactory quality and
 *      noninfringement. Keil extends you a royalty-free right to reproduce
 *      and distribute executable files created using this software for use
 *      on Philips LPC2xxx microcontroller devices only. Nothing else gives
 *      you the right to use this software.
 *
 *      Copyright (c) 2005-2006 Keil Software.
 *---------------------------------------------------------------------------*/

 //This Code follows the template of the basic HID Keil example

#include <LPC214X.H>                        /* LPC214x definitions */

#include "type.h"

#include "usb.h"
#include "usbcfg.h"
#include "usbhw.h"
#include "usbcore.h"

#include "demo.h"


BYTE InReport[8];                              /* HID Input Report - Keyboards have an 8 byte report*/
                                           
char bfr;
BYTE OutReport;                             /* HID Out Report */
                                            /*   Bit0,1,2: LEDs */
											//Defined in type.c
keyboard_st key;

/*
 *  Get HID Input Report -> InReport
 *  SW1, SW2, SW3 are pressed in a mutually exclusive manner.
 */
int i;
int fla;

int s;
int pos;


//The following code is used to initialize the LCD Module. This helps in easyb debugging. The code is taken from the Rhydolabz example.
//__________________________________________________________________________________________________________

#define RS  0x00020000              /* RS  - P1.17                          */
#define RW  0X00040000              /* R/W - P1.18                          */    
#define EN  0X00080000              /* E   - P1.19                          */
#define CLR 0X00FE0000
 
/****************************************************************************
* Function    : PORT_Initial                                                *
*                                                                           *
* Description : Function for PORT Initialization                            *
*                                                                           *
* Parameters  : None                                                        *
*****************************************************************************/
void Delay(unsigned long x)
{
	x=x-1;
	if(x==0)
		return;
}

void PORT_Initial(void)    
{
 IO1DIR  = 0x00FE0000;              /* LCD pins set as o/p                  */
 IO0DIR  = 0x002F0000;         
 			//
 PINSEL1 = 0x05000000;
 PINSEL2 = 0x00000000;
}
 
/****************************************************************************
* Function    : delay                                                       *
*                                                                           *
* Description : Function for delay                                          *
*                                                                           *
* Parameters  : x - delay time                                              *
*****************************************************************************/
int delay(unsigned int x)
{
 x=x*8000;
 while(x!=0)
 {
  x--;
 }
 return 0;
}
 
/****************************************************************************
* Function    : LCD_Command                                                 *
*                                                                           *
* Description : Function to give command to LCD                             *
*                                                                           *
* Parameters  : command - command to be given                               *
*****************************************************************************/
void LCD_Command(char command)
{
 int Temp;                           
 IO1CLR = CLR;                      /* Clearing the port pins               */
 IO1SET = EN;                       /* Enable pin high                      */
 IO1CLR = RS;                       /* RS=0 for command register            */
 IO1CLR = RW;                       /* R/W=0 for write                      */
 Temp   = (command & 0xF0) << 16;   /* Taking the first nibble of command   */
 IO1SET = IO1SET | Temp;            /* Writing it to data line              */
 delay(3);
 IO1CLR = EN;                       /* Enable pin low to give H-L pulse     */
}
 
/****************************************************************************
* Function    : LCD_Command1                                                *
*                                                                           *
* Description : Function to give command to LCD                             *
*                                                                           *
* Parameters  : command1 - command to be given                              *
*****************************************************************************/
void LCD_Command1(char command1)
{
 int Temp;
 IO1CLR = CLR;                      /* Clearing the port pins               */
 IO1SET = EN;                       /* Enable pin high                      */
 IO1CLR = RS;                       /* RS=0 for command register            */
 IO1CLR = RW;                       /* R/W=0 for write                      */
 Temp   = (command1 & 0xF0);        /* Taking the first nibble of command   */
 Temp   = Temp << 16;               /* Shift it 16 bits to left             */
 IO1SET = IO1SET | Temp;            /* Writing it to data line(P1.20-P1.23) */
 delay(3);
 IO1CLR = EN;                       /* Enable pin low to give H-L pulse     */
 
 IO1CLR = CLR;                      /* Clearing the port pins               */
 IO1SET = EN;                       /* Enable pin high                      */
 IO1CLR = RS;                       /* RS=0 for command register            */
 IO1CLR = RW;                       /* R/W=0 for write                      */
 Temp   = (command1 & 0x0F);        /* Taking the second nibble of command  */
 Temp   = Temp << 20;               /* Shift it 20 bits to left             */
 IO1SET = IO1SET | Temp;            /* Writing it to data line              */
 delay(3);
 IO1CLR = EN;                       /* Enable pin low to give H-L pulse     */
}
 
/****************************************************************************
* Function    : LCD_Data                                                    *
*                                                                           *
* Description : Function to display single character on LCD                 *
*                                                                           *
* Parameters  : data - Character to be displayed                            *
*****************************************************************************/
 void LCD_Data(char data)  
{
 int Temp;
 IO1CLR = CLR;                      /* Clearing the port pins               */
 IO1SET = EN;                       /* Enable pin high                      */
 IO1SET = RS;                       /* RS=1 for data register               */
 IO1CLR = RW;                       /* R/W=0 for write                      */
 Temp   = (data & 0xF0);            /* Taking the first nibble of data      */
 Temp   = Temp << 16;               /* Shift it 16 bits to left             */
 IO1SET = IO1SET | Temp;            /* Writing it to data line              */
 delay(3);
 IO1CLR = EN;                       /* Enable pin low to give H-L pulse     */
 
 IO1CLR = CLR;                      /* Clearing the port pins               */
 IO1SET = EN;                       /* Enable pin high                      */
 IO1SET = RS;                       /* RS=1 for data register               */
 IO1CLR = RW;                       /* R/W=0 for write                      */
 Temp   = (data & 0x0F);            /* Taking the second nibble of data     */
 Temp   = Temp << 20;               /* Shift it 20 bits to left             */
 IO1SET = IO1SET | Temp;            /* Writing it to data line              */
 delay(3);
 IO1CLR = EN;                       /* Enable pin low to give H-L pulse     */
}
 
/****************************************************************************
* Function    : LCD_String                                                  *
*                                                                           *
* Description : Function to display string on LCD                           *
*                                                                           *
* Parameters  : String to be displayed                                      *
*****************************************************************************/ 
void LCD_String(unsigned char *dat)
{
 while(*dat!='\0')                  /* Check for termination character      */
 {
  LCD_Data(*dat);                   /* Display the character on LCD         */
  dat++;                            /* Increment the pointer                */
 }
}
 
/****************************************************************************
* Function    : LCD_Init                                                    *
*                                                                           *
* Description : Function to initialize LCD                                  *
*                                                                           *
* Parameters  : None                                                        *
*****************************************************************************/ 
void LCD_Init(void)
{
 delay(15);
 LCD_Command(0x30);
 delay(10);    
 LCD_Command(0x30);
 delay(5);    
 LCD_Command(0x30);
 LCD_Command(0x20);
 LCD_Command1(0x28);
 LCD_Command1(0x01);                /* Clear display                        */
 LCD_Command1(0x06);                /* Auto increment                       */
 LCD_Command1(0x0C);                /* Cursor off                           */
}
//______________________________________________________________________________________________












void GetInReport (void) { //This is for Device(LPC2148) to Host PC reporting

    if (fla==1)
		{
			
			key.keycode[0] = bfr;	  //The modifier is set accordingly in the decode function
			fla=0;
		}
	else
		{
			key.modifier=0;
			key.keycode[0] = 0x00;	//Transmits nothing
		}
	
	memcpy(InReport,&key,sizeof key);
}


//More values will be added to this. The decoding can be extensive, as the USB Usage Tables are quite different from corresponding ASCII values.
char decode(char x)
{	
	 const char comp[]="abcdefghijklmnopqrstuvwxyz1234567890 ";
	 
	 const char codes[]={0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x2C};
	if((x>='a' && x<='z') || (x>='0' && x<='9'))
	{
		key.modifier=0x00;
	}
	else if(x>='A' && x<='Z')
	{	
		x=x+0x20;
		key.modifier=0x02; //Activates Left Control for Capital Letters
	}
	for(s=0;s<37;s++)
	{
		if(comp[s]==x)
		{
			pos=s;
			break;
		}
	}
	if(s<37)
		return(codes[pos]);
	else
		return(0x00);
	 
}
void SetOutReport (void) { //This is for Host PC to Device(LPC2148) reporting

//We leave this blank
}


/* Main Program */

int main (void) {

	
	char n;
	int count;
	char temp;
	USB_Init(); 
	
	PINSEL0 = 0X00050005;
	 VPBDIV=0x00; //To ensure PCLK is 12MHz
	//Initializing UART1
	U1LCR = 0X83;	   //Making DLAB bit 1.
 	U1FDR = 0X00000010;	
 	U1DLM = 0X01;	   //Required to set the baud rate of 2400
 	U1DLL = 0X39;	   //Assuming that PCLK of 12 Mhz is available
 	U1FCR = 0X05;	   
 	U1LCR = 0X03;                                             
															  
	key.modifier=0;
	key.reserved=0;
	key.keycode[0]=0;
	key.keycode[1]=0;
	key.keycode[2]=0;
	key.keycode[3]=0;
	key.keycode[4]=0;
	key.keycode[5]=0;
	fla=0;
  	PORT_Initial();                    /* Initialize port  for LCD                    */
 LCD_Init();
	                              /* USB Initialization */
  USB_Connect(TRUE);                        /* USB Connect */
	count = 0;
  while (1)
  {
   		
		while((U1LSR & 0X01) == 0X00);
		n=U1RBR;
		LCD_Command1(0x80); 
		LCD_Data(n);
		if(n>=0x20 && n<=0x7F)
		{
			if(count==0)
			{
				temp=n;
				count=count+1;
			}
			else if(n==temp)
			{
				count=count+1;
			}
			else
				count=0;
			if(count==10)
			{	
				bfr = decode(n);
				fla=1;//Will be written when the endpoint is polled
				
			}
		}
		else
		{	
			count=0;
		}
	}


  }
  
 
 	                                

